(**********************************************************************
 ** Required declarations
 **********************************************************************)

(** 
 ** Tokens are defined by the parser.  
 ** Tokens are the point of caommunication between the parser and the lexer.
 **)
structure Tokens = Tokens

exception Badnum

(** Must match that of the parser **)
type pos = int

(** Must have defnitions **)
type svalue = Tokens.svalue
type ('a,'b) token = ('a,'b) Tokens.token
type lexresult = (svalue, pos) token


(** Error processing stuff **)
  (** Fill in... **)

val pos = ref 0
val eof = fn () => Tokens.EOF(!pos,!pos)
val error = fn (e,l : int,_) =>
               TextIO.print ("line " ^ (Int.toString l) ^ ": " ^ e ^ "\n")

(** Make  token **)
  (** Fill in functions for making tokens  **)

%%
%header (functor CalcLexFun (structure Tokens: Calc_TOKENS));
alpha=[A-Za-z];
digit=[0-9];
ws = [\ \t];
identstart=[A-Za-z_];
identbody=[A-Za-z0-9_];
%%
\n       => (pos := !pos + 1; lex());
{ws}+    => (lex());
{digit}+ => (Tokens.NUM ((case (Int.fromString yytext) of 
                             SOME n => n | NONE => raise Badnum),
                         !pos,!pos));
";"      => (Tokens.STOP(!pos,!pos));
"\\stop"   => (Tokens.STOP(!pos,!pos));
"("      => (Tokens.LPAREN(!pos,!pos));
")"      => (Tokens.RPAREN(!pos,!pos));
"+"      => (Tokens.PLUS(!pos,!pos));
"fn"     => (Tokens.LAMBDA(!pos,!pos));
"lam"    => (Tokens.LAMBDA(!pos,!pos));
"."      => (Tokens.DOT(!pos,!pos));
"rep"    => (Tokens.REPEAT(!pos,!pos));
{identstart} ({identbody}+)? => (Tokens.VAR(yytext,!pos,!pos));
.        => (error ("calc: ignoring bad character " ^ yytext, !pos, !pos); lex());


